@@ -64,7 +64,8 @@ class App(object):
 
     def __init__(self, description, version, command_manager,
                  stdin=None, stdout=None, stderr=None,
-                 interactive_app_factory=InteractiveApp):
+                 interactive_app_factory=InteractiveApp,
+                 deferred_help=False):
         """Initialize the application.
         """
         self.command_manager = command_manager
@@ -72,6 +73,7 @@ class App(object):
         self.command_manager.add_command('complete', CompleteCommand)
         self._set_streams(stdin, stdout, stderr)
         self.interactive_app_factory = interactive_app_factory
+        self.deferred_help = deferred_help
         self.parser = self.build_option_parser(description, version)
         self.interactive_mode = False
         self.interpreter = None
@@ -86,10 +88,9 @@ class App(object):
             # works around a problem with Python 2.6 fixed in 2.7 and
             # later (http://hg.python.org/cpython/rev/e60ef17561dc/).
             lang, encoding = locale.getdefaultlocale()
-            encoding = (getattr(sys.stdout, 'encoding', None)
-                        or encoding
-                        or self.DEFAULT_OUTPUT_ENCODING
-                        )
+            encoding = (getattr(sys.stdout, 'encoding', None) or
+                        encoding or
+                        self.DEFAULT_OUTPUT_ENCODING)
             self.stdin = stdin or codecs.getreader(encoding)(sys.stdin)
             self.stdout = stdout or codecs.getwriter(encoding)(sys.stdout)
             self.stderr = stderr or codecs.getwriter(encoding)(sys.stderr)
@@ -144,13 +145,21 @@ class App(object):
             const=0,
             help='suppress output except warnings and errors',
         )
-        parser.add_argument(
-            '-h', '--help',
-            action=HelpAction,
-            nargs=0,
-            default=self,  # tricky
-            help="show this help message and exit",
-        )
+        if self.deferred_help:
+            parser.add_argument(
+                '-h', '--help',
+                dest='deferred_help',
+                action='store_true',
+                help="show this help message and exit",
+            )
+        else:
+            parser.add_argument(
+                '-h', '--help',
+                action=HelpAction,
+                nargs=0,
+                default=self,  # tricky
+                help="show this help message and exit",
+            )
         parser.add_argument(
             '--debug',
             default=False,
@@ -186,6 +195,19 @@ class App(object):
         root_logger.addHandler(console)
         return
 
+    def print_help_if_requested(self):
+        """Print help and exits if deferred help is enabled and requested.
+
+        '--help' shows the help message and exits:
+         * without calling initialize_app if not self.deferred_help (default),
+         * after initialize_app call if self.deferred_help,
+         * during initialize_app call if self.deferred_help and subclass calls
+           explicitly this method in initialize_app.
+        """
+        if self.deferred_help and self.options.deferred_help:
+            action = HelpAction(None, None, default=self)
+            action(self.parser, self.parser, None, None)
+
     def run(self, argv):
         """Equivalent to the main program for the application.
 
@@ -197,6 +219,7 @@ class App(object):
             self.configure_logging()
             self.interactive_mode = not remainder
             self.initialize_app(remainder)
+            self.print_help_if_requested()
         except Exception as err:
             if hasattr(self, 'options'):
                 debug = self.options.debug
