@@ -15,7 +15,7 @@ from cliff.command import Command
 from cliff.commandmanager import CommandManager
 
 
-def make_app():
+def make_app(**kwargs):
     cmd_mgr = CommandManager('cliff.tests')
 
     # Register a command that succeeds
@@ -38,6 +38,7 @@ def make_app():
               '1',
               cmd_mgr,
               stderr=mock.Mock(),  # suppress warning messages
+              **kwargs
               )
     return app, command
 
@@ -87,7 +88,7 @@ def test_clean_up_error():
     app.clean_up = mock.MagicMock(name='clean_up')
     app.run(['error'])
 
-    app.clean_up.assert_called_once()
+    app.clean_up.assert_called_once_with(mock.ANY, mock.ANY, mock.ANY)
     call_args = app.clean_up.call_args_list[0]
     assert call_args == mock.call(mock.ANY, 1, mock.ANY)
     args, kwargs = call_args
@@ -254,7 +255,8 @@ def test_output_encoding_default():
             )
 
     stdout = StringIO()
-    getdefaultlocale = lambda: ('ignored', 'utf-8')
+
+    getdefaultlocale = mock.Mock(return_value=('ignored', 'utf-8'))
 
     with mock.patch('sys.stdout', stdout):
         with mock.patch('locale.getdefaultlocale', getdefaultlocale):
@@ -281,7 +283,7 @@ def test_output_encoding_cliff_default():
             )
 
     stdout = StringIO()
-    getdefaultlocale = lambda: ('ignored', None)
+    getdefaultlocale = mock.Mock(return_value=('ignored', None))
 
     with mock.patch('sys.stdout', stdout):
         with mock.patch('locale.getdefaultlocale', getdefaultlocale):
@@ -309,7 +311,7 @@ def test_output_encoding_sys():
 
     stdout = StringIO()
     stdout.encoding = 'utf-8'
-    getdefaultlocale = lambda: ('ignored', 'utf-16')
+    getdefaultlocale = mock.Mock(return_value=('ignored', 'utf-16'))
 
     with mock.patch('sys.stdout', stdout):
         with mock.patch('locale.getdefaultlocale', getdefaultlocale):
@@ -336,7 +338,7 @@ def test_error_encoding_default():
             )
 
     stderr = StringIO()
-    getdefaultlocale = lambda: ('ignored', 'utf-8')
+    getdefaultlocale = mock.Mock(return_value=('ignored', 'utf-8'))
 
     with mock.patch('sys.stderr', stderr):
         with mock.patch('locale.getdefaultlocale', getdefaultlocale):
@@ -365,7 +367,7 @@ def test_error_encoding_sys():
     stdout = StringIO()
     stdout.encoding = 'utf-8'
     stderr = StringIO()
-    getdefaultlocale = lambda: ('ignored', 'utf-16')
+    getdefaultlocale = mock.Mock(return_value=('ignored', 'utf-16'))
 
     with mock.patch('sys.stdout', stdout):
         with mock.patch('sys.stderr', stderr):
@@ -376,6 +378,29 @@ def test_error_encoding_sys():
                 assert data == actual
 
 
+def _test_help(deferred_help):
+    app, _ = make_app(deferred_help=deferred_help)
+    with mock.patch.object(app, 'initialize_app') as init:
+        with mock.patch('cliff.help.HelpAction.__call__',
+                        side_effect=SystemExit(0)) as helper:
+            try:
+                app.run(['--help'])
+            except SystemExit:
+                pass
+            else:
+                raise Exception('Exception was not thrown')
+            assert helper.called
+        assert init.called == deferred_help
+
+
+def test_help():
+    _test_help(False)
+
+
+def test_deferred_help():
+    _test_help(True)
+
+
 def test_unknown_cmd():
     app, command = make_app()
     assert app.run(['hell']) == 2
